/*	-*- C++ -*-
 * Copyright (C) 2001-2002,  Christof Meerwald <cmeerw@web.de>
 * http://xpcom2corba.sunsite.dk/
 */

/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License
 * at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and
 * limitations under the License.
 *
 * The Original Code is the XPCOM-CORBA Bridge package.
 *
 * The Initial Developer of the Original Code is Christof Meerwald.
 * Portions created by Christof Meerwald are Copyright (C) 2001
 * Christof Meerwald.  All Rights Reserved.
 *
 * Contributor(s): Christof Meerwald <cmeerw@web.de> (original author)
 */
#ifndef HEADER__CORBA2XPCOM__INCLUDED
#define HEADER__CORBA2XPCOM__INCLUDED

#include <CORBA.h>
#include <nsISupports.h>
#ifndef MOZILLA_INTERNAL_API
#include <nsStringAPI.h>
#include <nsMemory.h>
#else
#include <nsString.h>
#endif

#include <wchar.h>

inline void corba2xpcom(CORBA::Boolean corba_val, PRBool *xpcom_val)
{
  *xpcom_val = corba_val;
}

inline void corba2xpcom(CORBA::Octet corba_val, PRUint8 *xpcom_val)
{
  *xpcom_val = corba_val;
}

inline void corba2xpcom(CORBA::Char corba_val, PRInt8 *xpcom_val)
{
  *xpcom_val = corba_val;
}

inline void corba2xpcom(CORBA::UShort corba_val, PRUint16 *xpcom_val)
{
  *xpcom_val = corba_val;
}

inline void corba2xpcom(CORBA::Short corba_val, PRInt16 *xpcom_val)
{
  *xpcom_val = corba_val;
}

inline void corba2xpcom(CORBA::ULong corba_val, PRUint32 *xpcom_val)
{
  *xpcom_val = corba_val;
}

inline void corba2xpcom(CORBA::Long corba_val, PRInt32 *xpcom_val)
{
  *xpcom_val = corba_val;
}

inline void corba2xpcom(CORBA::Double corba_val, double *xpcom_val)
{
  *xpcom_val = corba_val;
}

inline void corba2xpcom(CORBA::Float corba_val, float *xpcom_val)
{
  *xpcom_val = corba_val;
}

inline void corba2xpcom(const char *corba_val, char **xpcom_val)
{
  *xpcom_val =
    (char *) nsMemory::Clone(corba_val, sizeof (char) * (strlen(corba_val) + 1));
}

inline void corba2xpcom(const CORBA::WChar *corba_val, PRUnichar **xpcom_val)
{
#ifdef WIN32
  const size_t len = wcslen(corba_val);

  *xpcom_val =
    (PRUnichar *) nsMemory::Clone(corba_val, sizeof (PRUnichar) * (len + 1));
#else
  // On some platforms (we may need to fix the #if to get this right) omniORB
  // default is UCS4 but Mozilla always uses UTF16. So we need to drop the
  // upper bytes. Care is needed, because this code is compiled with
  // -fshort-wchar to match Mozilla.
  const PRUint32* p = (const PRUint32*)corba_val;
  while (*p != 0)
    p++;
  const size_t len = (p - ((const PRUint32*)corba_val));
  *xpcom_val = (PRUnichar*)nsMemory::Alloc(2 * (len + 1));
  
  p = (const PRUint32*)corba_val;
  size_t i;
  for (i = 0; i < len; i++)
    ((PRUint16*)(*xpcom_val))[i] = (PRUint16)(*p++);
  ((PRUint16*)(*xpcom_val))[i] = 0;
#endif
}

inline void corba2xpcom(const char *corba_val, nsACString &xpcom_val)
{
  xpcom_val.Assign(corba_val, strlen(corba_val));
}

inline void corba2xpcom(const CORBA::WChar *corba_val, nsACString &xpcom_val)
{
  // TODO
  // const size_t len = wcslen(corba_val);
}

#endif
